@@ -76,12 +76,6 @@ NewObject <- function(type.name, ...)
 
 UseFunction <- function(fn.name, ...)
 {
-  #cat("Objects visible in UseFunction:\n")
-  #print(sapply(sys.frames(), function(x) ls(x)))
-  #cat("Call stack for UseFunction:\n")
-  #sapply(sys.calls(), function(x) print(x))
-  #cat("\n")
-  #fn.name <- deparse(substitute(fn))
   fn <- get(fn.name, inherits=TRUE)
   result <- NULL
   raw.args <- list(...)
@@ -92,11 +86,12 @@ UseFunction <- function(fn.name, ...)
   for (v in vs)
   {
     full.args <- fill_args(raw.args, v)
+    if (is.null(full.args)) next
     full.type <- get_type(fn,v$type.index)
     if (!check_types(full.type, full.args)) next
     if (is.null(v$guard)) { matched.fn <- v$def; break }
     gout <- do.call(v$guard, full.args)
-    if (length(gout) > 0 && gout) { matched.fn <- v$def; break }
+    if (!is.na(gout) && length(gout) > 0 && gout) { matched.fn <- v$def; break }
   }
   if (is.null(matched.fn))
     stop(use_error(.ERR_USE_FUNCTION,fn.name,raw.args))
@@ -110,8 +105,9 @@ UseFunction <- function(fn.name, ...)
     if (return.type == '.lambda.r_UNIQUE')
     {
       act <- paste(class(result), collapse=', ')
-      if (class(result) %in% sapply(raw.args, class)) {
-        msg <- sprintf("Expected unique return type but found '%s' for",act)
+      first <- class(result)[1]
+      if (first %in% sapply(raw.args, class)) {
+        msg <- sprintf("Expected unique return type but found '%s' for",first)
         stop(use_error(msg,fn.name,raw.args))
       }
     }
@@ -128,17 +124,22 @@ UseFunction <- function(fn.name, ...)
 }
 
 
+has_ellipsis <- function(tree) {
+  '...' %in% tree$args$token
+}
+
 fill_args <- function(raw.args, tree)
 {
   if (is.null(tree$args)) return(list())
 
+  has.ellipsis <- has_ellipsis(tree)
   tree$args <- tree$args[tree$args$token != '...',]
   defaults <- tree$args$default
 
   # This is for unnamed arguments
   if (is.null(names(raw.args)))
   {
-    if (length(raw.args) == length(defaults)) return(raw.args)
+    if (length(raw.args) >= length(defaults)) return(raw.args)
     ds <- defaults[(length(raw.args)+1):length(defaults)]
     vs <- lapply(ds, function(x) eval(parse(text=x)))
     names(vs) <- NULL
@@ -146,6 +147,8 @@ fill_args <- function(raw.args, tree)
   }
   else
   {
+    if (! has.ellipsis && any(! names(raw.args) %in% c("",tree$args$token))) 
+      return(NULL)
     ds <- lapply(defaults, function(x) x)
     names(ds) <- tree$args$token
     shim <- tree$args$token[1:length(raw.args)]
